(*
 * Coq code for the paper
 *
 *       A Syntactic Approach to Foundational Proof-Carrying Code
 *                            
 *
 *   by Zhong Shao, Valery Trifonov, Stefan Monnier, Nadeem A. Hamid, 
 *                          and Zhaozhong Ni
 *
 * (for Coq version 7.0)
 *)


(***************************************************************************)
(* Utilities for recursive type encoding *)
(***************************************************************************)
Require EqNat.

(* substitution *)
Fixpoint subst_aux [i:nat; t:(OmegaL i)]
  : (j:nat) i=(S j) -> (OmegaL j) -> (OmegaL j)
  := <[i:nat; _:(OmegaL i)] (j:nat) i=(S j) -> (OmegaL j) -> (OmegaL j)>
     Cases t of
       (varL n) => [j:nat; _:((S n)=(S j)); e':(OmegaL j)] e'
     | (liftL n t0) => 
       	       	   [j:nat; p:((S n)=(S j)); _:(OmegaL j)]
		   (eq_rec ? n OmegaL t0 j (eq_add_S n j p))
     | (inttyL) => [j:nat; _:(O=(S j)); e':(OmegaL j)] e' (* not reached *)
     | (codetyL n G A) =>
       	       	   [j:nat; p:(n=(S j)); e':(OmegaL j)]
		   (codetyL j (substmap_aux n G j p e') A)
     | (tuptyL n L bl) =>
       	       	   [j:nat; p:(n=(S j)); e':(OmegaL j)]
		   (tuptyL j (substlist_aux n L j p e') bl)
     | (rectyL n t0) =>
       	       	   [j:nat; p:(n=(S j)); e':(OmegaL j)]
		   (rectyL j (subst_aux (S n) t0 (S j)
		                     (f_equal ? ? S n (S j) p)
				     (liftL j e')))
     end
with substmap_aux [i:nat; M:(OmegaL_Map i)]
  : (j:nat) i=(S j) -> (OmegaL j) -> (OmegaL_Map j)
  := <[i:nat; _:(OmegaL_Map i)] (j:nat)i=(S j) -> (OmegaL j) -> (OmegaL_Map j)>
     Cases M of
       (memptyL n) => 
      	       	  [j:nat; _:(n=(S j)); _:(OmegaL j)] (memptyL j)
     | (melemL n r t M0) =>
       	       	  [j:nat; p:(n=(S j)); e':(OmegaL j)]
		  (melemL j r
		              (subst_aux n t j p e')
			      (substmap_aux n M0 j p e'))
     end
with substlist_aux [i:nat; L:(OmegaL_List i)]
  : (j:nat) i=(S j) -> (OmegaL j) -> (OmegaL_List j)
  := <[i:nat; _:(OmegaL_List i)](j:nat)i=(S j)->(OmegaL j)->(OmegaL_List j)>
     Cases L of
       (nilL n) => [j:nat; _:(n=(S j)); _:(OmegaL j)] (nilL j)
     | (consL n t L0) =>
       	       	   [j:nat; p:(n=(S j)); e':(OmegaL j)]
		   (consL j
		          (subst_aux n t j p e')
			  (substlist_aux n L0 j p e'))
     end
.

Definition SubstL := [n:nat; T:(OmegaL (S n))]
      	       	     (subst_aux (S n) T n (refl_equal nat (S n))).


Definition RUnfold : (OmegaL (S O)) -> (OmegaL O)
  := [T:(OmegaL (S O))] (SubstL O T (rectyL O T)).




(* conversion to the real Omega *)
Fixpoint RUnlift [T:(OmegaL O)] : Omega
  := ((<[n:nat]n=O -> Omega>
      Cases T of
        (varL i) => [p:((S i)=O)]
      	       	    (False_rec Omega 
		               (eq_ind nat (S i)
			                   [e:nat]
					   Cases e of O => False | _ => True
					   end I O p))
      | (liftL i T0) =>
      	       	    [p:((S i)=O)]
      	       	    (False_rec Omega 
		               (eq_ind nat (S i)
			                   [e:nat]
					   Cases e of O => False | _ => True
					   end I O p))
      | (inttyL) => [_:(O=O)] intty
      | (rectyL i T0) => [p:(i=O)]
      	       	       	 (recty (eq_rec nat (S i) OmegaL T0 (S O)
			                             (eq_S i O p)))
      | (codetyL i M A) => [p:(i=O)] (codety (RUnlift_Map 
      	       	       	       	       	       (eq_rec nat i OmegaL_Map M O p))
      	       	       	       	       	        A)
      | (tuptyL i L bl) => [p:(i=O)] (tupty (RUnlift_List 
     	       	       	       	       	      (eq_rec nat i OmegaL_List L O p))
      	       	       	       	       	      bl)
      end) (refl_equal nat O))
with RUnlift_Map [M:(OmegaL_Map O)] : (Map Reg Omega)
  := ((<[n:nat]n=O -> (Map Reg Omega)>
      Cases M of
      	(memptyL i) => [_:(i=O)] (mempty Reg Omega)
      | (melemL i r T M0) => 
      	       	       [p:(i=O)] 
		       (melem Reg Omega r (RUnlift 
		                            (eq_rec nat i OmegaL T O p))
					  (RUnlift_Map 
					    (eq_rec nat i OmegaL_Map M0 O p)))
      end) (refl_equal nat O))
with RUnlift_List [L:(OmegaL_List O)] : (list Omega)
  := ((<[n:nat]n=O -> (list Omega)>
      Cases L of
      	(nilL i) => [_:(i=O)] (nil Omega)
      | (consL i T L0) =>
      	       	    [p:(i=O)]
		    (cons (RUnlift (eq_rec nat i OmegaL T O p))
		          (RUnlift_List (eq_rec nat i OmegaL_List L0 O p)))
      end) (refl_equal nat O))
.

(***************************************************************************)
(***************************************************************************)
(***************************************************************************)

(* samples:
Definition T := (rectyL (S O) (tuptyL (S (S O)) 
      	       	       	       	     (consL (S (S O)) 
      	       	       	       	       	    (varL (S O)) 
      	       	       	       	       	    (nilL (S (S O))))
      	       	       	       	     (cons true (nil ?)))).
*)





