(*
 * Coq code for the paper
 *
 *       A Syntactic Approach to Foundational Proof-Carrying Code
 *                            
 *
 *   by Zhong Shao, Valery Trifonov, Stefan Monnier, Nadeem A. Hamid, 
 *                          and Zhaozhong Ni
 *
 * (for Coq version 7.0 - 7.3)
 *)


(***************************************************************************)
(* Definition and lemmas of registers in TIS *)
(***************************************************************************)

(* This file must be compiled using coqc to tisreg.vo because it is 
   required as such by tis.v, ie. type in the shell:
coqc tisreg.v
 *)

Inductive _Reg : Set := _r0 : _Reg | _r1 : _Reg | _r2 : _Reg | _r3 : _Reg 
      	       	     |  _r4 : _Reg | _r5 : _Reg | _r6 : _Reg | _r7 : _Reg 
      	       	     |  _r8 : _Reg | _r9 : _Reg | _r10: _Reg | _r11 : _Reg 
      	       	     |  _r12 : _Reg | _r13 : _Reg | _r14 : _Reg
                     |  _r15: _Reg | _r16 : _Reg | _r17 : _Reg | _r18 : _Reg 
      	       	     |  _r19 : _Reg | _r20: _Reg | _r21 : _Reg | _r22 : _Reg 
      	       	     |  _r23 : _Reg | _r24 : _Reg
                     |  _r25: _Reg | _r26 : _Reg | _r27 : _Reg | _r28 : _Reg 
      	       	     |  _r29 : _Reg | _r30: _Reg | _r31 : _Reg.

Definition nat2_reg := [n:nat] Cases n of O => _r0 | (S O) => _r1 | (S (S O)) => _r2 | (S (S (S O))) => _r3 | (S (S (S (S O)))) => _r4 | (S (S (S (S (S O))))) => _r5 | (S (S (S (S (S (S O)))))) => _r6 | (S (S (S (S (S (S (S O))))))) => _r7 | (S (S (S (S (S (S (S (S O)))))))) => _r8 | (S (S (S (S (S (S (S (S (S O))))))))) => _r9 | (S (S (S (S (S (S (S (S (S (S O)))))))))) => _r10 | (S (S (S (S (S (S (S (S (S (S (S O))))))))))) => _r11 | (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))) => _r12 | (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))) => _r13 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))) => _r14 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))) => _r15 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))) => _r16 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))) => _r17 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))) => _r18 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))) => _r19 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))) => _r20 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))) => _r21 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))) => _r22 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))) => _r23 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))))) => _r24 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))))) => _r25 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))))))) => _r26 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))))))) => _r27 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))))))))) => _r28 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))))))))) => _r29 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))))))))))) => _r30 | (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))))))))))) => _r31 | _ => _r0 end.


Definition _reg2nat := [r:_Reg] Cases r of  _r0 => O |  _r1 => (S O) |  _r2 => (S (S O)) |  _r3 => (S (S (S O))) |  _r4 => (S (S (S (S O)))) |  _r5 => (S (S (S (S (S O))))) |  _r6 => (S (S (S (S (S (S O)))))) |  _r7 => (S (S (S (S (S (S (S O))))))) |  _r8 => (S (S (S (S (S (S (S (S O)))))))) |  _r9 => (S (S (S (S (S (S (S (S (S O))))))))) |  _r10 => (S (S (S (S (S (S (S (S (S (S O)))))))))) |  _r11 => (S (S (S (S (S (S (S (S (S (S (S O))))))))))) |  _r12 => (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))) |  _r13 => (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))) |  _r14 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))) |  _r15 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))) |  _r16 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))) |  _r17 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))) |  _r18 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))) |  _r19 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))) |  _r20 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))) |  _r21 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))) |  _r22 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))) |  _r23 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))) |  _r24 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))))) |  _r25 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))))) |  _r26 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))))))) |  _r27 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))))))) |  _r28 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))))))))) |  _r29 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))))))))) |  _r30 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O)))))))))))))))))))))))))))))) |  _r31 => (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S (S O))))))))))))))))))))))))))))))) end.


Definition beq_reg := [r1,r2:_Reg] Cases r1 r2 of  _r0 _r0 => true  |  _r1 _r1 => true  |  _r2 _r2 => true  |  _r3 _r3 => true  |  _r4 _r4 => true  |  _r5 _r5 => true  |  _r6 _r6 => true  |  _r7 _r7 => true  |  _r8 _r8 => true  |  _r9 _r9 => true  |  _r10 _r10 => true  |  _r11 _r11 => true  |  _r12 _r12 => true  |  _r13 _r13 => true  |  _r14 _r14 => true  |  _r15 _r15 => true  |  _r16 _r16 => true  |  _r17 _r17 => true  |  _r18 _r18 => true  |  _r19 _r19 => true  |  _r20 _r20 => true  |  _r21 _r21 => true  |  _r22 _r22 => true  |  _r23 _r23 => true  |  _r24 _r24 => true  |  _r25 _r25 => true  |  _r26 _r26 => true  |  _r27 _r27 => true  |  _r28 _r28 => true  |  _r29 _r29 => true  |  _r30 _r30 => true  |  _r31 _r31 => true  | _ _  => false end.


Lemma beq_reg_neq_false : (r,r':_Reg) ~r=r' -> (beq_reg r r')=false.
Double Induction 1 2.
Intros.
Absurd ~_r0=_r0; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r1=_r1; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r2=_r2; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r3=_r3; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r4=_r4; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r5=_r5; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r6=_r6; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r7=_r7; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r8=_r8; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r9=_r9; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r10=_r10; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r11=_r11; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r12=_r12; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r13=_r13; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r14=_r14; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r15=_r15; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r16=_r16; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r17=_r17; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r18=_r18; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r19=_r19; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r20=_r20; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r21=_r21; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r22=_r22; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r23=_r23; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r24=_r24; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r25=_r25; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r26=_r26; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r27=_r27; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r28=_r28; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r29=_r29; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r30=_r30; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Unfold beq_reg; Auto. Intros.
Absurd ~_r31=_r31; Auto. 
Save.



Lemma beq_reg_eq_true : (r:_Reg) (beq_reg r r)=true.
Proof.
Intros.
Induction r.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Auto.
Save.


(***************************************************************************)


Lemma eqorneq_tisreg : (r,r':_Reg) r=r' \/ ~r=r'.
Proof.
Intros.
Compare r r'.
Left.
Auto.
Right.
Auto.
Save.
