(*
 * Coq code for the paper
 *
 *       A Syntactic Approach to Foundational Proof-Carrying Code
 *                            
 *
 *   by Zhong Shao, Valery Trifonov, Stefan Monnier, Nadeem A. Hamid, 
 *                          and Zhaozhong Ni
 *
 * (for Coq version 7.3)
 *)


(***************************************************************************)
(* Utility lemmas for translation from FTAL to machine state *)
(***************************************************************************)

Require talreg.
Require tisreg.

(* register translation *)
Definition TrRegF : Reg -> _Reg
  := [regi:Reg]
     Cases regi of
       	       r0 => _r0
	     | r1 => _r1    | r2 => _r2     | r3 => _r3
	     | r4 => _r4    | r5 => _r5     | r6 => _r6
	     | r7 => _r7    | r8 => _r8     | r9 => _r9
	     | r10 => _r10  | r11 => _r11   | r12 => _r12
	     | r13 => _r13  | r14 => _r14   | r15 => _r15
	     | r16 => _r16  | r17 => _r17   | r18 => _r18
	     | r19 => _r19  | r20 => _r20   | r21 => _r21
	     | r22 => _r22  | r23 => _r23   | r24 => _r24
	     | r25 => _r25  | r26 => _r26   | r27 => _r27
	     | r28 => _r28  | r29 => _r29   | r30 => _r30 end.


Definition InvTrRegF : (r:_Reg)(~r=_r31) -> Reg
:= [r:_Reg; H:(~r=_r31)]
 (_Reg_rec [_:_Reg]~_=_r31->Reg [_:(~_r0=_r31)]r0 [_:(~_r1=_r31)]r1
   [_:(~_r2=_r31)]r2 [_:(~_r3=_r31)]r3 [_:(~_r4=_r31)]r4
   [_:(~_r5=_r31)]r5 [_:(~_r6=_r31)]r6 [_:(~_r7=_r31)]r7
   [_:(~_r8=_r31)]r8 [_:(~_r9=_r31)]r9 [_:(~_r10=_r31)]r10
   [_:(~_r11=_r31)]r11 [_:(~_r12=_r31)]r12 [_:(~_r13=_r31)]r13
   [_:(~_r14=_r31)]r14 [_:(~_r15=_r31)]r15 [_:(~_r16=_r31)]r16
   [_:(~_r17=_r31)]r17 [_:(~_r18=_r31)]r18 [_:(~_r19=_r31)]r19
   [_:(~_r20=_r31)]r20 [_:(~_r21=_r31)]r21 [_:(~_r22=_r31)]r22
   [_:(~_r23=_r31)]r23 [_:(~_r24=_r31)]r24 [_:(~_r25=_r31)]r25
   [_:(~_r26=_r31)]r26 [_:(~_r27=_r31)]r27 [_:(~_r28=_r31)]r28
   [_:(~_r29=_r31)]r29 [_:(~_r30=_r31)]r30
   [H0:(~_r31=_r31)]
    (False_rec Reg
      [H1:=H0]
       ([H2:=[_:(_r31=_r31->False)](H0 (refl_equal _Reg _r31))]
         [H3:(~_r31=_r31)](H2 H3) H1)) r H).



(* lemmas *)

Lemma trreg_neq_31 : (r:Reg; _r:_Reg) (TrRegF r)=_r -> ~_r=_r31.
Proof.
Intros.
Induction r.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Compute in H. Rewrite <- H. Discriminate.
Save.

Lemma trreg_inv : (r:Reg; _r:_Reg; D:(~_r=_r31))  
       	       	      (TrRegF r)=_r -> r=(InvTrRegF _r D).
Proof.
Double Induction 1 2.
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. Intros; Inversion H. 
Intros. Absurd ~_r31=_r31; Auto.
Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. Intros; Inversion H. 
Intros; Compute; Auto.
Intros; Inversion H. 
Save.



Lemma trregf_eq_eq : (r1,r2:Reg; r:_Reg) (TrRegF r1)=r -> (TrRegF r2)=r
      	       	       	       	       	-> r1=r2.
Induction r.
Intros. Cut ~_r0=_r31. Intros. Cut r1=(InvTrRegF _r0 H1); Intros. Cut r2=(InvTrRegF _r0 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r1=_r31. Intros. Cut r1=(InvTrRegF _r1 H1); Intros. Cut r2=(InvTrRegF _r1 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r2=_r31. Intros. Cut r1=(InvTrRegF _r2 H1); Intros. Cut r2=(InvTrRegF _r2 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r3=_r31. Intros. Cut r1=(InvTrRegF _r3 H1); Intros. Cut r2=(InvTrRegF _r3 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r4=_r31. Intros. Cut r1=(InvTrRegF _r4 H1); Intros. Cut r2=(InvTrRegF _r4 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r5=_r31. Intros. Cut r1=(InvTrRegF _r5 H1); Intros. Cut r2=(InvTrRegF _r5 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r6=_r31. Intros. Cut r1=(InvTrRegF _r6 H1); Intros. Cut r2=(InvTrRegF _r6 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r7=_r31. Intros. Cut r1=(InvTrRegF _r7 H1); Intros. Cut r2=(InvTrRegF _r7 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r8=_r31. Intros. Cut r1=(InvTrRegF _r8 H1); Intros. Cut r2=(InvTrRegF _r8 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r9=_r31. Intros. Cut r1=(InvTrRegF _r9 H1); Intros. Cut r2=(InvTrRegF _r9 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r10=_r31. Intros. Cut r1=(InvTrRegF _r10 H1); Intros. Cut r2=(InvTrRegF _r10 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r11=_r31. Intros. Cut r1=(InvTrRegF _r11 H1); Intros. Cut r2=(InvTrRegF _r11 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r12=_r31. Intros. Cut r1=(InvTrRegF _r12 H1); Intros. Cut r2=(InvTrRegF _r12 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r13=_r31. Intros. Cut r1=(InvTrRegF _r13 H1); Intros. Cut r2=(InvTrRegF _r13 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r14=_r31. Intros. Cut r1=(InvTrRegF _r14 H1); Intros. Cut r2=(InvTrRegF _r14 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r15=_r31. Intros. Cut r1=(InvTrRegF _r15 H1); Intros. Cut r2=(InvTrRegF _r15 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r16=_r31. Intros. Cut r1=(InvTrRegF _r16 H1); Intros. Cut r2=(InvTrRegF _r16 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r17=_r31. Intros. Cut r1=(InvTrRegF _r17 H1); Intros. Cut r2=(InvTrRegF _r17 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r18=_r31. Intros. Cut r1=(InvTrRegF _r18 H1); Intros. Cut r2=(InvTrRegF _r18 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r19=_r31. Intros. Cut r1=(InvTrRegF _r19 H1); Intros. Cut r2=(InvTrRegF _r19 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r20=_r31. Intros. Cut r1=(InvTrRegF _r20 H1); Intros. Cut r2=(InvTrRegF _r20 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r21=_r31. Intros. Cut r1=(InvTrRegF _r21 H1); Intros. Cut r2=(InvTrRegF _r21 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r22=_r31. Intros. Cut r1=(InvTrRegF _r22 H1); Intros. Cut r2=(InvTrRegF _r22 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r23=_r31. Intros. Cut r1=(InvTrRegF _r23 H1); Intros. Cut r2=(InvTrRegF _r23 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r24=_r31. Intros. Cut r1=(InvTrRegF _r24 H1); Intros. Cut r2=(InvTrRegF _r24 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r25=_r31. Intros. Cut r1=(InvTrRegF _r25 H1); Intros. Cut r2=(InvTrRegF _r25 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r26=_r31. Intros. Cut r1=(InvTrRegF _r26 H1); Intros. Cut r2=(InvTrRegF _r26 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r27=_r31. Intros. Cut r1=(InvTrRegF _r27 H1); Intros. Cut r2=(InvTrRegF _r27 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r28=_r31. Intros. Cut r1=(InvTrRegF _r28 H1); Intros. Cut r2=(InvTrRegF _r28 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r29=_r31. Intros. Cut r1=(InvTrRegF _r29 H1); Intros. Cut r2=(InvTrRegF _r29 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r30=_r31. Intros. Cut r1=(InvTrRegF _r30 H1); Intros. Cut r2=(InvTrRegF _r30 H1); Intros. Compute in H2. Compute in H3. Rewrite H2; Rewrite H3; Auto. Apply trreg_inv; Auto. Apply trreg_inv; Auto. Apply trreg_neq_31 with r1; Auto. 
Intros. Cut ~_r31=_r31. Intros. Absurd ~_r31=_r31; Auto. Apply trreg_neq_31 with r1; Auto. 
Save.


Lemma trregf_neq_neq : (r1,r2:Reg; r,r':_Reg) 
      	       	      (TrRegF r1)=r -> (TrRegF r2)=r' ->
		      ~r=r' -> ~r1=r2. 
Proof.
Intros.
Cut r1=r2\/~r1=r2.
Intros.
Inversion_clear H2.
Rewrite H3 in H.
Cut r=r'.
Intros.
Rewrite H2 in H1.
Absurd ~r'=r'.
Auto.
Auto.
Rewrite H in H0.
Auto.
Auto.
Apply eqorneq_reg.
Save.
